<?php
// $Id

/**
 * @file
 * Page callback file for the blog module.
 */

/**
 * Handles displaying a list of blogs in a couple of ways, either a striaght listing
 * of all blogs or a list of the blogs grouped by the user that created them
 */
function everyblog_list_blogs() {
	$output ='';
	
	// determine what order the blogs should be listed in
	if (isset($_GET['orderby'])) {
		$order_by = strtolower($_GET['orderby']);
	}
	if ( $order_by == 'recent') {
		$order_by == 'recent';
	}
	else {
		$order_by = 'alpha';
	}
	
	// The path might be "blogs/user" or "blogs/category" or just "blogs"
	if (func_num_args() == 1) {
		$view = func_get_arg(0);
		$view = split('\?', $view);
		$view = strtolower($view[0]);
	}
	else {
		$view = '';
	}
	
	// Show blogs grouped by user
	if( $view == 'user') {
		$is_user_view = TRUE;
		list($title, $breadcrumb, $blogs, $error_message) = everyblog_get_blogs_grouped_by_user($order_by);
	}
	// Show blogs grouped by category
	else if ($view == 'category') {
		$is_category_view = TRUE;
		list($title, $breadcrumb, $blogs, $error_message) = everyblog_get_blogs_grouped_by_category($order_by);
	}
	// Show a regular listing of all blogs
	else {
		$is_regular_view = TRUE;
		list($title, $breadcrumb, $blogs, $error_message) = everyblog_get_all_blogs($order_by);
	}
	
	// setup the title
	drupal_set_title($title);
	
	// Setup a basic breadcrumb
	drupal_set_breadcrumb(array(
		l(t('Home'), NULL), 
		l(t('Blogs'), 'blogs'),
		$breadcrumb));
		
	// all the user to sort the blogs
	switch ($order_by) {
		case 'recent':
			$output .= '['.l(t('Order Alphabetically'), "blogs/$view", array('query'=>'orderby=alpha')).']';
			break;
		
		case 'alpha':		
		default:
			$output .= '['.l(t('Order by Most Recent Post'), "blogs/$view", array('query'=>'orderby=recent')).']';
	}
	$output .= "<br>\n";
		
	// display the blogs
	if (count($blogs) > 0) {
		if ($is_regular_view) {
			$output .= theme('item_list', $blogs);
		}
		elseif ($is_user_view || $is_category_view) {
			foreach ($blogs as $groupname=>$group) {
				$output .= $groupname;
				$output .= theme('item_list', $group);
			}
		}
	}
	else {
		drupal_set_message($error_message, 'status');
	}

	// give the user an option to see the blogs grouped by user or by category
	if ($is_regular_view) {		
		$output .= '<br>';
		$output .= l(t('View Blogs Grouped by User'), 'blogs/user');
		$output .= '<br>';
		$output .= l(t('View Blogs Grouped by Category'), 'blogs/category');
	}
			
	drupal_add_feed(url('blogs/feed'), t('RSS - All Blogs'));
	
	return $output;
}

function everyblog_get_all_blogs($order_by) {
	$title = t('All Blogs');
	$breadcrumb = t('All Blogs');
	
	if ($order_by == 'alpha') {
		$query = 'SELECT blog_id, blog_name FROM {everyblogs} ORDER BY blog_name';
	}
	elseif ($order_by == 'recent') {
		$query = 'SELECT * FROM (SELECT blog_id, blog_name, created FROM {everyblogs} JOIN ({everyblogs_to_nid}) USING (blog_id) JOIN ({node}) USING (nid) ORDER BY created DESC) AS t GROUP BY blog_id ORDER BY created DESC';
	}

	$result = db_query($query);
	while($blog = db_fetch_object($result)) {
		$blogs[] = l($blog->blog_name, 'blog/'._everyblog_encode($blog->blog_name))
			. (variable_get('everyblog_show_datetime', FALSE) ?
				' (<i>' . date('D, M j, Y G:i', $blog->created) . '</i>)' : '');
	}
	return array($title, $breadcrumb, $blogs, $error_message);
}

function everyblog_get_blogs_grouped_by_category($order_by) {
	$title = t('All Blogs Grouped by Category');
	$breadcrumb = t('By Category');
	 
	// Get a list of categories blogs are in
	if ($order_by == 'alpha') {
		$category_list = db_query('SELECT tid FROM {everyblogs} JOIN {term_data} USING (tid) GROUP BY tid ORDER BY name');
	}
	elseif ($order_by == 'recent') {
		$category_list = db_query('SELECT * FROM (SELECT {everyblogs}.tid, created FROM {everyblogs} JOIN ({everyblogs_to_nid}) USING (blog_id) JOIN ({node}) USING (nid) ORDER BY created DESC) AS t GROUP BY tid ORDER BY created DESC');
	}
	
	
	// For each user who has blogs
	while ($category = db_fetch_object($category_list)) {
		// load the user so we can get their username
		$category = taxonomy_get_term($category->tid);
		$group = l($category->name, "blogs/category/{$category->tid}");
		
		// get a list of blogs under this category
		if ($order_by == 'alpha') {
			$query = 'SELECT blog_id, blog_name FROM {everyblogs} WHERE tid=%s ORDER BY blog_name';
		}
		elseif ($order_by == 'recent') {
			$query = 'SELECT * FROM (SELECT blog_id, blog_name, created FROM {everyblogs} JOIN ({everyblogs_to_nid}) USING (blog_id) JOIN ({node}) USING (nid) WHERE tid=%s ORDER BY created DESC) AS t GROUP BY blog_id ORDER BY created DESC';
		}
		$result = db_query($query, $category->tid);
		while ($blog = db_fetch_object($result)) {
			$blogs[$group][] = l($blog->blog_name, 'blog/'._everyblog_encode($blog->blog_name))
				. (variable_get('everyblog_show_datetime', FALSE) ?
					' (<i>' . date('D, M j, Y G:i', $blog->created) . '</i>)' : '');
		}

		$error_message = t('There are no blogs in this category yet.');
	}
	
	return array($title, $breadcrumb, $blogs, $error_message);
}
function everyblog_get_blogs_grouped_by_user($order_by) {
	$title = t('All Blogs Grouped By User');
	$breadcrumb = t('By User');
	
	// Get a list of users who have made blogs
	if ($order_by == 'alpha') {
		$users_result = db_query('SELECT uid FROM {everyblogs} JOIN {users} USING (uid) GROUP BY uid ORDER BY name');
	}
	elseif ($order_by == 'recent') {
		$users_result = db_query('SELECT * FROM (SELECT {everyblogs}.uid, created FROM {everyblogs} JOIN ({everyblogs_to_nid}) USING (blog_id) JOIN ({node}) USING (nid) ORDER BY created DESC) AS t GROUP BY uid ORDER BY created DESC');
	}
	
	// For each user who has blogs
	while ($u = db_fetch_object($users_result)) {
		// load the user so we can get their username
		$user = user_load($u->uid);
		$group = l($user->name, "blogs/user/{$u->uid}");
		
		// get a list of blogs that the user has
		if ($order_by == 'alpha') {
			$query = 'SELECT blog_id, blog_name FROM {everyblogs} WHERE uid=%s ORDER BY blog_name';
		}
		elseif ($order_by == 'recent') {
			$query = 'SELECT * FROM (SELECT blog_id, blog_name, created FROM {everyblogs} JOIN ({everyblogs_to_nid}) USING (blog_id) JOIN ({node}) USING (nid) WHERE {everyblogs}.uid=%s ORDER BY created DESC) AS t GROUP BY blog_id ORDER BY created DESC';
		}
		
		$result = db_query($query, $user->uid);
		while ($blog = db_fetch_object($result)) {
			$blogs[$group][] = l($blog->blog_name, 'blog/'._everyblog_encode($blog->blog_name))
				. (variable_get('everyblog_show_datetime', FALSE) ?
					' (<i>' . date('D, M j, Y G:i', $blog->created) . '</i>)' : '');
		}
		
		$error_message = t('There are no blogs by this user yet.');
	}
	
	return array($title, $breadcrumb, $blogs, $error_message);
}

/**
 * Lists all the blogs by a particular user
 *
 * @param $uid mixed if its a number its assumed to be a uid otherwise its taken to be the username
 */
function everyblog_list_blogs_by_user($uid) {
	$output ='';
	
	if (is_numeric($uid)) {
		$user = user_load($uid);
		if ($user == FALSE) {
			drupal_set_message(
				t('No matching user found. Please check for typing errors in the URL.'),
				'error');
			return '';
		}
	}
	else {
		$user = user_load(array('name'=>$uid));
		if ($user == FALSE) {
			drupal_set_message(
					t("No matching user found for username '@uid' Please check for typing errors in the URL.",
					array('@uid'=>$uid)
					),
				'error');
			return '';
		}
	}
	$uid = $user->uid;
	drupal_set_title($title=t('Blogs by ') . $user->name);
	drupal_set_breadcrumb(array(
		l(t('Home'), NULL),
		l(t('Blogs'), 'blogs'),
		l(t('User'), 'blogs/user'),
		$user->name,
		));
		
	$query = 'SELECT blog_id, blog_name FROM {everyblogs} WHERE uid='.$uid.' ORDER BY blog_name';
	$result = db_query($query);
	while($blog = db_fetch_object($result)) {
		$blogs[] = l($blog->blog_name, 'blog/'._everyblog_encode($blog->blog_name))
			. (variable_get('everyblog_show_datetime', FALSE) ?
				' (<i>' . date('D, M j, Y G:i', $blog->created) . '</i>)' : '');
	}
	$output .= theme('item_list', $blogs);
	
	drupal_add_feed(url('blogs/user/'. $user->name .'/feed'), t('RSS - Blogs by !username', array('!username' => $user->name)));
	return $output;	
}

/**
 * Lists all the blogs by a particular user
 *
 * @param $uid mixed if its a number its assumed to be a uid otherwise its taken to be the username
 */
function everyblog_list_blogs_by_category($tid) {
	$output ='';
	
	// If the URL part is numeric then its a taxonomy term id
	if (is_numeric($tid)) {		
		$category = taxonomy_get_term($tid);
		if ($category == FALSE) {
			drupal_set_message(t('No matching category was found. Please check for typing errors in the URL.'), 'error');
			return '';
		}
	}
	// The URL part is a category name, so look it up
	else {
		$category = taxonomy_get_term_by_name($tid);
		if ($category == FALSE) {
			drupal_set_message(
					t("No matching category was found for '@tid'. Please check for typing errors in the URL.",
					array('@tid'=>$tid)
					), 
				'error');
			return '';
		}
		$category = $category[0];
	}
	$tid = $category->tid;
	drupal_set_title($title=t('Blogs in ') . $category->name);
	drupal_set_breadcrumb(array(
		l(t('Home'), NULL),
		l(t('Blogs'), 'blogs'),
		l(t('Category'), 'blogs/category'),
		$category->name,
		));
		
	$query = 'SELECT blog_id, blog_name FROM {everyblogs} WHERE tid='.$tid.' ORDER BY blog_name';
	$result = db_query($query);
	while($blog = db_fetch_object($result)) {
		$blogs[] = l($blog->blog_name, 'blog/'._everyblog_encode($blog->blog_name))
			. (variable_get('everyblog_show_datetime', FALSE) ?
				' (<i>' . date('D, M j, Y G:i', $blog->created) . '</i>)' : '');
	}
	$output .= theme('item_list', $blogs);
	
	drupal_add_feed(url('blogs/category/'. $category->name .'/feed'), t('RSS - Blogs in !category', array('!category' => $category->name)));
	return $output;
}

/**
 * This encodes strings in such a way that they are both human readable and URL safe.
 * Spaces are replaced by underscores '_' then the string is urlencoded.
 *
 * @param $str string the string to encode in a safe way
 */
function _everyblog_encode($str) {
	return urlencode(str_replace(' ', '_', $str));
}

/**
 * Displays recent blog entries of a given blog.
 *
 * @param $blog_id mixed if its a number then its taken to be the blogs id, otherwise its
 *		taken to be the blogs name, either URL encoded or with spaces replaced by
 * 		underscores '_'
 */
function everyblog_page($blog_id) {
	// Load the blog id, since $blog_id can be an actual blog id or the blog name
	if ( !is_numeric($blog_id) ) {
		$blog_name = urldecode($blog_id);
		$blog_name = str_replace('_', ' ', $blog_name);
		$result = db_query("SELECT blog_id FROM {everyblogs} WHERE blog_name LIKE \"%s\"", $blog_name);
		$blog_id = db_fetch_object($result);
		$blog_id = $blog_id->blog_id;
	}
	
	// Get the blog title
	$query = 'SELECT * FROM {everyblogs} WHERE blog_id=%s';
	$blog = db_fetch_object(db_query($query, $blog_id));
	$user = user_load($blog->uid);
	$term = taxonomy_get_term($blog->tid);
	
	drupal_set_title($title = $blog->blog_name);
	drupal_set_breadcrumb(array(
		l(t('Home'), NULL),
		l(t('Blogs'), 'blogs'),
		t('By !user in !category', array(
			'!user'=>l($user->name, 'blogs/user/' . $user->name),
			'!category'=>l($term->name, 'blogs/category/' . $term->name)
			)),
		$blog->blog_name,
		));


	$items = array();

	if (($blog->uid == $user->uid) && user_access('create own blog entries')) {
		$items[] = l(t('Post new blog entry.'), "node/add/everyblog" , 
			array('query'=>array('blog'=>$blog->blog_id)));
	}

	$output .= theme('item_list', $items);

	if ($user->uid != $blog->uid) {
		$published_catch = 'n.status = 1 AND';
	}
	
	$query = <<<EOF
SELECT n.nid, n.sticky, n.created 
	FROM 
		{node} AS n,
		{everyblogs_to_nid} AS e_to_n
	WHERE
		$published_catch
		n.type = 'everyblog' AND
		n.nid = e_to_n.nid AND
		e_to_n.blog_id = '$blog_id'
	ORDER BY n.sticky DESC, n.created DESC
EOF;

	$result = pager_query($query, 
		variable_get('default_nodes_main', 10), 0, NULL, $account->uid);
	$has_posts = FALSE;
	
	while ($node = db_fetch_object($result)) {
		$output .= node_view(node_load($node->nid), 1);
		$has_posts = TRUE;
	}
	
	if ($has_posts) {
		$output .= theme('pager', NULL, variable_get('default_nodes_main', 10));
	}
	else {
		if ($blog->uid == $user->uid) {
			drupal_set_message(t('You have not created any blog entries.'));
		}
		else {
			drupal_set_message(t('No blog entries have been created in this blog.'));
		}
	}
	drupal_add_feed(url('blog/'. $blog->blog_name .'/feed'), t('RSS - !title', array('!title' => $title)));

	return $output;
}

function everyblog_nodeapi(&$node, $op, $a3, $a4) {
	global $user;
	
	if ($op == 'alter') {
		if ($node->type == 'everyblog' && $user->uid == $node->uid && $node->status == '0') {
			$node->title = t('[Unpublished]') . ' ' . $node->title;
		}
	}
}

/**
 * Menu callback; displays a Drupal page containing recent blog entries of all users.
 */
function everyblog_page_last() {
	global $user;

	$output = '';
	$items = array();

	if (user_access('edit own blog')) {
		$items[] = l(t('Create new blog entry.'), "node/add/blog");
	}

	$output = theme('item_list', $items);

	$result = pager_query(db_rewrite_sql("SELECT n.nid, n.created FROM {node} n WHERE n.type = 'blog' AND n.status = 1 ORDER BY n.sticky DESC, n.created DESC"), variable_get('default_nodes_main', 10));
	$has_posts = FALSE;

	while ($node = db_fetch_object($result)) {
		$output .= node_view(node_load($node->nid), 1);
		$has_posts = TRUE;
	}
	
	if ($has_posts) {
		$output .= theme('pager', NULL, variable_get('default_nodes_main', 10));
	}
	else {
		drupal_set_message(t('No blog entries have been created.'));
	}
	drupal_add_feed(url('blog/feed'), t('RSS - blogs'));

	return $output;
}

/**
 * Menu callback; displays an RSS feed containing recent blog entries of a given user.
 */
function everyblog_feed_user($uid) {
	// load the user
	if (is_numeric($uid)) {
		$user = user_load($uid);
	}
	else {
		$user = user_load(array('name'=>$uid));
	}
	if ($user == FALSE) {
		$channel['title'] = t('No blogs by @username', array('@username' => $uid));
		$channel['link'] = url("/", array('absolute' => TRUE));
		$items = array();
		node_feed($items, $channel);
		return;
	}
	$uid = $user->uid;

	// Set the channel information
	$channel['title'] = t('Blogs by @username', array('@username' => $user->name));
	$channel['link'] = url("blogs/user/$uid", array('absolute' => TRUE));
	
	$query = <<<EOF
SELECT e.blog_id, e.blog_name, n.nid, n.created 
FROM {everyblogs} AS e, {everyblogs_to_nid} AS etn, {node} AS n 
WHERE e.uid=$uid AND e.blog_id = etn.blog_id AND etn.nid = n.nid AND n.status = 1 ORDER BY n.created DESC
EOF;
	
	// Run the query 
	$result = db_query_range($query, 0, variable_get('feed_default_items', 10));

	// get the feed items
	while($entry = db_fetch_object($result)) {
		$entries[] = $entry->nid;
	}
	node_feed($entries, $channel);
}

/**
 * Menu callback; displays an RSS feed containing recent blog entries in a given category
 */
function everyblog_feed_category($tid) {
	// If the URL part is numeric then its a taxonomy term id
	if (is_numeric($tid)) {		
		$category = taxonomy_get_term($tid);
	}
	// The URL part is a category name, so look it up
	else {
		$category = taxonomy_get_term_by_name($tid);
		$category = $category[0];
	}
	if ($category === FALSE) {
		$channel['title'] = t('No category @tid', array('@tid'=>$tid));
		$channel['link'] = url("/", array('absolute' => TRUE));
		$items = array();
		node_feed($items, $channel);
		return;
	}
	$tid = $category->tid;

	// Set the channel information
	$channel['title'] = t('Blogs in @category', array('@category' => $category->name));
	$channel['link'] = url("blogs/category/$tid", array('absolute' => TRUE));
	
	$query = <<<EOF
SELECT e.blog_id, e.blog_name, n.nid, n.created 
FROM {everyblogs} AS e, {everyblogs_to_nid} AS etn, {node} AS n 
WHERE e.tid=$tid AND e.blog_id = etn.blog_id AND etn.nid = n.nid AND n.status = 1 ORDER BY n.created DESC
EOF;
	
	// Run the query 
	$result = db_query_range($query, 0, variable_get('feed_default_items', 10));

	// get the feed items
	while($entry = db_fetch_object($result)) {
		$entries[] = $entry->nid;
	}
	node_feed($entries, $channel);
}

/**
 * Menu callback; displays an RSS feed containing recent blog entries in a given blog.
 */
function everyblog_feed_blog($blog_id) {
	// Load the blog id, since $blog_id can be an actual blog id or the blog name
	if ( !is_numeric($blog_id) ) {
		$blog_name = urldecode($blog_id);
		$blog_name = str_replace('_', ' ', $blog_name);
		$result = db_query("SELECT blog_id FROM {everyblogs} WHERE blog_name LIKE \"%s\"", $blog_name);
		$blog_id = db_fetch_object($result);
		$blog_id = $blog_id->blog_id;
	}
	// Get the blog title
	$query = 'SELECT * FROM {everyblogs} WHERE blog_id=%s';
	$blog = db_fetch_object(db_query($query, $blog_id));

	// Set the channel information
	$channel['title'] = t('@blogname: Recent Entries', array('@blogname' => $blog->blog_name));
	$channel['link'] = url("blog/{$blog->blog_name}", array('absolute' => TRUE));
	
	$query = <<<EOF
SELECT e.blog_id, e.blog_name, n.nid, n.created 
FROM {everyblogs} AS e, {everyblogs_to_nid} AS etn, {node} AS n 
WHERE e.blog_id=$blog_id AND e.blog_id = etn.blog_id AND etn.nid = n.nid AND n.status = 1 ORDER BY n.created DESC
EOF;
	
	// Run the query 
	$result = db_query_range($query, 0, variable_get('feed_default_items', 10));

	// get the feed items
	while($entry = db_fetch_object($result)) {
		$entries[] = $entry->nid;
	}
	node_feed($entries, $channel);
}

/**
 * Menu callback; displays an RSS feed containing recent blog entries of all blogs.
 */
function everyblog_feed_all() {
	$result = db_query_range(db_rewrite_sql("SELECT n.nid, n.created FROM {node} n WHERE n.type = 'everyblog' AND n.status = 1 ORDER BY n.created DESC"), 0, variable_get('feed_default_items', 10));
	$channel['title'] = variable_get('site_name', 'Drupal') .' blogs';
	$channel['link'] = url('blog', array('absolute' => TRUE));

	$items = array();
	while ($row = db_fetch_object($result)) {
		$items[] = $row->nid;
	}

	node_feed($items, $channel);
}